// Regular Expression Tester v1.0
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.

#region Using declarations

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;
using System.Data;
using System.Net;
using System.Text.RegularExpressions;

#endregion

namespace Regular_Expression_Tester {

	public class RegularExpressionTester : System.Windows.Forms.Form {

		#region Variables, initialization code, and Main

		private System.Windows.Forms.Label label1;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button btnMatch;
		private System.Windows.Forms.TextBox txtRegExp;
		private System.Windows.Forms.TextBox txtSource;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox txtMatchCount;
		private System.Windows.Forms.Button btnClearSourceText;
		private System.Windows.Forms.Button btnNextMatch;
		private System.Windows.Forms.Button btnPreviousMatch;
		private System.Windows.Forms.CheckBox checkSingleLine;
		private System.Windows.Forms.Button btnHelp;
		private RegularExpressionReference help;
		private int[] iMatchIndices, iMatchLengths;
		private ArrayList aGroupCollections;
		private System.Windows.Forms.Button btnLoadSourceText;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TextBox txtCapturedGroups;
		private int iCurrentMatch;

		public RegularExpressionTester() {
			InitializeComponent();
			iMatchIndices = new int[0];
			iMatchLengths = new int[0];
			aGroupCollections = new ArrayList();
			help = new RegularExpressionReference();
		}

		[STAThread]
		static void Main() {
			Application.Run(new RegularExpressionTester());
		}

		#endregion

		#region Uninteresting Windows Form Designer generated code

		private void InitializeComponent() {
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(RegularExpressionTester));
			this.label1 = new System.Windows.Forms.Label();
			this.txtRegExp = new System.Windows.Forms.TextBox();
			this.btnMatch = new System.Windows.Forms.Button();
			this.txtSource = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.btnLoadSourceText = new System.Windows.Forms.Button();
			this.txtMatchCount = new System.Windows.Forms.TextBox();
			this.btnNextMatch = new System.Windows.Forms.Button();
			this.btnClearSourceText = new System.Windows.Forms.Button();
			this.btnPreviousMatch = new System.Windows.Forms.Button();
			this.checkSingleLine = new System.Windows.Forms.CheckBox();
			this.btnHelp = new System.Windows.Forms.Button();
			this.label3 = new System.Windows.Forms.Label();
			this.txtCapturedGroups = new System.Windows.Forms.TextBox();
			this.SuspendLayout();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(112, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Regular Expression:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtRegExp
			// 
			this.txtRegExp.Anchor = ((System.Windows.Forms.AnchorStyles) (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
															| System.Windows.Forms.AnchorStyles.Right)));
			this.txtRegExp.Location = new System.Drawing.Point(8, 32);
			this.txtRegExp.Multiline = true;
			this.txtRegExp.Name = "txtRegExp";
			this.txtRegExp.Size = new System.Drawing.Size(402, 40);
			this.txtRegExp.TabIndex = 0;
			// 
			// btnMatch
			// 
			this.btnMatch.Location = new System.Drawing.Point(8, 80);
			this.btnMatch.Name = "btnMatch";
			this.btnMatch.Size = new System.Drawing.Size(64, 23);
			this.btnMatch.TabIndex = 2;
			this.btnMatch.Text = "Match";
			this.btnMatch.Click += new System.EventHandler(this.btnMatch_Click);
			// 
			// txtSource
			// 
			this.txtSource.Anchor = ((System.Windows.Forms.AnchorStyles) ((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
															| System.Windows.Forms.AnchorStyles.Left)
															| System.Windows.Forms.AnchorStyles.Right)));
			this.txtSource.HideSelection = false;
			this.txtSource.Location = new System.Drawing.Point(8, 144);
			this.txtSource.Multiline = true;
			this.txtSource.Name = "txtSource";
			this.txtSource.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.txtSource.Size = new System.Drawing.Size(402, 216);
			this.txtSource.TabIndex = 8;
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 120);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(100, 23);
			this.label2.TabIndex = 7;
			this.label2.Text = "Source Text:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// btnLoadSourceText
			// 
			this.btnLoadSourceText.Anchor = ((System.Windows.Forms.AnchorStyles) ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnLoadSourceText.Location = new System.Drawing.Point(266, 115);
			this.btnLoadSourceText.Name = "btnLoadSourceText";
			this.btnLoadSourceText.Size = new System.Drawing.Size(64, 23);
			this.btnLoadSourceText.TabIndex = 6;
			this.btnLoadSourceText.Text = "Load";
			this.btnLoadSourceText.Click += new System.EventHandler(this.btnLoadSourceText_Click);
			// 
			// txtMatchCount
			// 
			this.txtMatchCount.Anchor = ((System.Windows.Forms.AnchorStyles) (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
															| System.Windows.Forms.AnchorStyles.Right)));
			this.txtMatchCount.Location = new System.Drawing.Point(80, 80);
			this.txtMatchCount.Name = "txtMatchCount";
			this.txtMatchCount.ReadOnly = true;
			this.txtMatchCount.Size = new System.Drawing.Size(258, 20);
			this.txtMatchCount.TabIndex = 3;
			this.txtMatchCount.Text = "Ready to match.";
			// 
			// btnNextMatch
			// 
			this.btnNextMatch.Anchor = ((System.Windows.Forms.AnchorStyles) ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnNextMatch.Location = new System.Drawing.Point(378, 80);
			this.btnNextMatch.Name = "btnNextMatch";
			this.btnNextMatch.Size = new System.Drawing.Size(32, 23);
			this.btnNextMatch.TabIndex = 5;
			this.btnNextMatch.Text = "-->";
			this.btnNextMatch.Click += new System.EventHandler(this.btnNext_Click);
			// 
			// btnClearSourceText
			// 
			this.btnClearSourceText.Anchor = ((System.Windows.Forms.AnchorStyles) ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnClearSourceText.Location = new System.Drawing.Point(346, 115);
			this.btnClearSourceText.Name = "btnClearSourceText";
			this.btnClearSourceText.Size = new System.Drawing.Size(64, 23);
			this.btnClearSourceText.TabIndex = 7;
			this.btnClearSourceText.Text = "Clear";
			this.btnClearSourceText.Click += new System.EventHandler(this.btnClearSourceText_Click);
			// 
			// btnPreviousMatch
			// 
			this.btnPreviousMatch.Anchor = ((System.Windows.Forms.AnchorStyles) ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnPreviousMatch.Location = new System.Drawing.Point(346, 80);
			this.btnPreviousMatch.Name = "btnPreviousMatch";
			this.btnPreviousMatch.Size = new System.Drawing.Size(32, 23);
			this.btnPreviousMatch.TabIndex = 4;
			this.btnPreviousMatch.Text = "<--";
			this.btnPreviousMatch.Click += new System.EventHandler(this.btnPreviousMatch_Click);
			// 
			// checkSingleLine
			// 
			this.checkSingleLine.Anchor = ((System.Windows.Forms.AnchorStyles) ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.checkSingleLine.CheckAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.checkSingleLine.Location = new System.Drawing.Point(298, 7);
			this.checkSingleLine.Name = "checkSingleLine";
			this.checkSingleLine.Size = new System.Drawing.Size(112, 24);
			this.checkSingleLine.TabIndex = 1;
			this.checkSingleLine.Text = "Single-Line Mode";
			// 
			// btnHelp
			// 
			this.btnHelp.Anchor = System.Windows.Forms.AnchorStyles.Bottom;
			this.btnHelp.Location = new System.Drawing.Point(101, 469);
			this.btnHelp.Name = "btnHelp";
			this.btnHelp.Size = new System.Drawing.Size(216, 23);
			this.btnHelp.TabIndex = 10;
			this.btnHelp.Text = "Regular Expression Symbol Reference";
			this.btnHelp.Click += new System.EventHandler(this.btnHelp_Click);
			// 
			// label3
			// 
			this.label3.Anchor = ((System.Windows.Forms.AnchorStyles) ((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.label3.Location = new System.Drawing.Point(8, 368);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(100, 23);
			this.label3.TabIndex = 16;
			this.label3.Text = "Captured Groups:";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtCapturedGroups
			// 
			this.txtCapturedGroups.Anchor = ((System.Windows.Forms.AnchorStyles) (((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)
															| System.Windows.Forms.AnchorStyles.Right)));
			this.txtCapturedGroups.HideSelection = false;
			this.txtCapturedGroups.Location = new System.Drawing.Point(8, 392);
			this.txtCapturedGroups.Multiline = true;
			this.txtCapturedGroups.Name = "txtCapturedGroups";
			this.txtCapturedGroups.ReadOnly = true;
			this.txtCapturedGroups.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.txtCapturedGroups.Size = new System.Drawing.Size(402, 72);
			this.txtCapturedGroups.TabIndex = 9;
			// 
			// RegularExpressionTester
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(418, 499);
			this.Controls.Add(this.txtCapturedGroups);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.btnHelp);
			this.Controls.Add(this.checkSingleLine);
			this.Controls.Add(this.btnPreviousMatch);
			this.Controls.Add(this.btnClearSourceText);
			this.Controls.Add(this.btnNextMatch);
			this.Controls.Add(this.txtMatchCount);
			this.Controls.Add(this.txtSource);
			this.Controls.Add(this.txtRegExp);
			this.Controls.Add(this.btnLoadSourceText);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.btnMatch);
			this.Controls.Add(this.label1);
			this.Icon = ((System.Drawing.Icon) (resources.GetObject("$this.Icon")));
			this.Name = "RegularExpressionTester";
			this.Text = "Regular Expression Tester";
			this.ResumeLayout(false);
			this.PerformLayout();

		}

		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#endregion

		#region Button-click handlers

		private void btnLoadSourceText_Click(object sender, System.EventArgs e) {
			// Load the source text from a text file of the user's choosing.
			OpenFileDialog d = new OpenFileDialog();
			if (d.ShowDialog() == DialogResult.OK) {
				txtSource.Text = "";
				StreamReader s = new StreamReader(d.FileName);
				txtSource.Text = s.ReadToEnd();
				s.Close();
			}
		}

		private void btnClearSourceText_Click(object sender, System.EventArgs e) {
			// Erase the source text.
			txtSource.Text = "";
			txtSource.Refresh();
		}

		private void btnMatch_Click(object sender, System.EventArgs e) {
			// Match the given regular expression to the source text.
			aGroupCollections = new ArrayList();
			try {
				Regex r = new Regex(txtRegExp.Text, checkSingleLine.CheckState == CheckState.Checked ? RegexOptions.Singleline : RegexOptions.Multiline);
				Match matches = r.Match(txtSource.Text);
				if (matches.Success == false) {
					txtMatchCount.Text = "No matches.";
					iMatchIndices = new int[0];
					iMatchLengths = new int[0];
				}
				else {
					// count matches
					Match matchcount = matches;
					int iTotal = 0;
					while (matchcount.Success == true)  {
						matchcount = matchcount.NextMatch();
						iTotal++;
					}
					iMatchIndices = new int[iTotal];
					iMatchLengths = new int[iTotal];
					iCurrentMatch = 0;
					for (int i = 0; i <iTotal; i++) {
						iMatchIndices[i] = matches.Index;
						iMatchLengths[i] = matches.Length;
						aGroupCollections.Add(matches.Groups);
						matches = matches.NextMatch();
					}
				}
				SelectCurrent();
			} catch (Exception ex) {
				MessageBox.Show("Error in regular expression: " + ex.Message, "Error in regular expression", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}
		}

		private void btnNext_Click(object sender, System.EventArgs e) {
			// Move to the next match, and update the text panel by caling SelectCurrent.
			if (iCurrentMatch < iMatchIndices.Length - 1) {
				iCurrentMatch++;
				SelectCurrent();
			}
		}

		private void btnPreviousMatch_Click(object sender, System.EventArgs e) {
			// Move to the previous match, and update the text panel by caling SelectCurrent.
			if (iCurrentMatch > 0) {
				iCurrentMatch--;
				SelectCurrent();
			}
		}

		private void btnHelp_Click(object sender, System.EventArgs e) {
			// Display the help file to the right of the main window.
			if (help.bAlive == false)
				help = new RegularExpressionReference();
			help.MoveForm(this.Location.X + this.Width, this.Location.Y);
		}

		#endregion

		#region Other functions

		private void SelectCurrent() {
			// Select the current match text in the text panel.
			if (iMatchIndices.Length == 0) {
				txtSource.Select(0, 0);
				txtCapturedGroups.Text = "";
			} else {
				txtSource.Focus();
				txtSource.Select(iMatchIndices[iCurrentMatch], iMatchLengths[iCurrentMatch]);
				txtSource.ScrollToCaret();
				txtMatchCount.Text = "Match " + (iCurrentMatch + 1) + " of " + iMatchIndices.Length;
				// display group captures
				txtCapturedGroups.Text = "";
				GroupCollection group = (GroupCollection) aGroupCollections[iCurrentMatch];
				for (int i = 0; i < group.Count; i++)
					txtCapturedGroups.Text += "Group " + i + ": " + group[i].Value + "\r\n";
			}
		}

		#endregion

	}
}
